/*
    Delete Current Playing File plugin for Amarok 2
    Copyright (C) 2009 Marcelo Juchem

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

Importer.loadQtBinding("qt.core");

function deleteCurrentPlayingFile()
{
	try
	{
		var activeIndex = Amarok.Playlist.activeIndex();
		var file = new QFile(Amarok.Playlist.filenames()[activeIndex]);

		if(!file.exists())
		{
			Amarok.alert("File already removed or non existent", "information");
		}
		else if(Amarok.alert("Cofirm deletion of file: " + file.fileName(), "warningContinueCancel") == 5)
		{
			var trackPosition = -1;

			try
			{
				trackPosition = Amarok.Engine.trackPositionMs();
			}
			catch(e)
			{
				try
				{
					trackPosition = Amarok.Engine.trackPosition() * 1000;
				}
				catch(e2)
				{
					trackPosition = -1;
				}
			}

			Amarok.Engine.Stop(true);

			if(file.remove())
			{
				var playlistFiles = Amarok.Playlist.filenames();

				for(var i = playlistFiles.length; i-- != 0; )
				{
					if(playlistFiles[i] == file.fileName())
					{
						if(i < activeIndex)
						{
							--activeIndex;
						}

						Amarok.Playlist.removeByIndex(i);
					}
				}

				var playlistLength = Amarok.Playlist.totalTrackCount();

				if(playlistLength > 0)
				{
					Amarok.Playlist.playByIndex(
						activeIndex >= playlistLength
							? playlistLength - 1
							: activeIndex
					);
					
				}
			}
			else
			{
				Amarok.alert("Could not delete " + file.fileName(), "error");

				if(file.exists())
				{
					Amarok.Playlist.playByIndex(activeIndex);

					if(trackPosition > 0)
					{
						Amarok.Engine.Seek(trackPosition);
					}
				}
			}
		}
	}
	catch(e)
	{
		Amarok.alert(e ? e : "Uknown reason", "error");
	}
}

Amarok.Window.addToolsMenu("deleteCurrentPlayingFile", "Delete current playing file...", "archive-remove");
Amarok.Window.ToolsMenu.deleteCurrentPlayingFile['triggered()'].connect(null, deleteCurrentPlayingFile);
