/**************************************************************************
*   Amarok 2 Ultimate Lyrics                                              *
*                                                                         *
*   Copyright                                                             *
*   (C) 2009-2010 Vladimir Brkic <vladimir_brkic@yahoo.com>               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
**************************************************************************/
//--------------------------------------------------------------------------

Importer.loadQtBinding( "qt.core" );
Importer.loadQtBinding( "qt.network" );
//Importer.loadQtBinding( "qt.xml" );
//Importer.loadQtBinding( "qt.gui");
//Importer.loadQtBinding( "qt.uitools");

//---------- translations --------------------------------------------------
const lang = QLocale.system().name();
//const lang = (new QLocale()).name();
//const lang = QTextCodec.codecForLocale();
//const lang = "es";

function qsTranslate(context,st){
	return QCoreApplication.translate(context,st);
}

var translator = new QTranslator();
translator.load("locale."+lang,Amarok.Info.scriptPath()+"/translations");
QCoreApplication.installTranslator(translator);

//QScriptEngine.installTranslatorFunctions();

//var res = new QResource();
//res.registerResource(Amarok.Info.scriptPath() + "/icons.rcc");

//---------- default settings ----------------------------------------------
//	After changing the file you must restore defaults 
const DEFAULTS = {
	version : "2.0",

	rotating_mode : true,			//	reload lyrics will change source site

	//	1. "Xml"	- plain text
	//	2. "Html"	- content inherited from site (italic, bold are considered)
	display_mode : "Xml",			//	"Xml" or "Html"

	display_title : true,
	display_url : false,

	max_size_html : 256,			//	html input size limit in KB
	max_size_lyrics : 8,			//	result size limit in KB

	multi_lang_wikia: true,

	omit_parenthesises: true,
	omit_brackets: true,
	omit_track_number: true,
	omit_the: false,
	replace_amp: true,

	gsearch : 'lyrics',
	gsearch_no : 10,
	gsearch_filter_artist : true,
	gsearch_filter_url : "hostname",	//	"no filter", "full url" or "hostname"

	gsearch_filter_expr : "lyr|wiki|sing|song|loud|jamendo|stixoi|tekst|vagalume|zeneszoveg",	//	url must contain one of the strings to pass filter
	gsearch_domains : ["lyrics","wikia.com"],

	cache: "no cache",			//	"file cache" or "no cache"

	file_cache_path : Amarok.Info.scriptPath() + "/cache",
//	file_cache_path : "../.kde/share/apps/amarok/ultimate_lyrics_cache",
	file_cache_folders : "{artist}/{title}/{site}{hash}",
	file_cache_expires : 7*24*60*60*1000,	//	7 days
	file_cache_auto_remove : false,

	//	sites are arranged in order they will be checked
	//	if default priority doesn't satisfy your needs you can change it by reordering the list
	//	if you discover a site causing problems (silly content or wrong lyrics all the time) just disable the site by setting "search" to false
	sites: {
		//	fully supported
		"lyrics.wikia.com":	{"search":true,	gSearchable:true},
		"lyricsplugin.com":	{"search":true,	gSearchable:true},
		"lyricstime.com":	{"search":true,	gSearchable:true},
		"lyricsreg.com":	{"search":true,	gSearchable:true},
		"lyricsmania.com":	{"search":true,	gSearchable:true},
		"metrolyrics.com":	{"search":true, gSearchable:true},
		"seeklyrics.com":	{"search":true,	gSearchable:true},
		"azlyrics.com":		{"search":true,	gSearchable:true},
		"jamendo.com":		{"search":false,gSearchable:true},
		"mp3lyrics.org":	{"search":false,gSearchable:true},
		"songlyrics.com":	{"search":false,gSearchable:true},
		"darklyrics.com":	{"search":false,gSearchable:false},
		"lyricsmode.com":	{"search":false,gSearchable:true},
		"elyrics.net":		{"search":false,gSearchable:true},
		"lyricsdownload.com":	{"search":false,gSearchable:true},
		"lyrics.com":		{"search":false,gSearchable:true},
//		"lyriki.com":		{"search":false,gSearchable:true},			//	poor source
		"lyricsbay.com":	{"search":false,gSearchable:true},
		"directlyrics.com":	{"search":false,gSearchable:true},
		"loudson.gs":		{"search":false,gSearchable:true},
		"teksty.org":		{"search":false,gSearchable:true},
		"tekstowo.pl (Polish translations)":	{"search":false,gSearchable:true},
		"vagalume.uol.com.br":	{"search":false,gSearchable:true},
		"vagalume.uol.com.br (Portuguese translations)": {"search":false,gSearchable:true},
		//	not automatically searchable
		"lyricsfreak.com":	{"search":false,gSearchable:true,searchLock:true},
		"sing365.com":		{"search":false,gSearchable:true,searchLock:true},
		"allreggaelyrics.com":	{"search":false,gSearchable:true,searchLock:true},
		"stixoi.info (Greek songs)":	{"search":false,gSearchable:true,searchLock:true},
		"zeneszoveg.hu":	{"search":false,gSearchable:true,searchLock:true},

		"Universal":		{"search":false,gSearchable:false,searchLock:true},
		//	suggestions
//		"NotFound" : 		{"search":true,	gSearchable:false,searchLock:true},
		"google.com":		{"search":true, gSearchable:false}
	}
}

//--------- debug ---------------------------------------------------------
const DEBUG = Math.floor(DEFAULTS.version*100)!=DEFAULTS.version*100;
function alert(msg){
	if (DEBUG)
		Amarok.alert(msg);
	Amarok.debug(/*"Ultimate Lyrics: " + */msg);
//	Amarok.Lyrics.showLyricsError(msg);
}

//--------------------------------------------------------------------------
Importer.include( "json2.js" );
Importer.include( "string.js" );
Importer.include( "cache.js" );
Importer.include( "sites.js" );
Importer.include( "lyrics.js" );
Importer.include( "configure.js" );
Importer.include( "patch.js" );

var config = new SettingsControl("settings",DEFAULTS);
var settings;

const NEW_INSTALLATION = DEFAULTS.version != Amarok.Script.readConfig("version","");
Amarok.Script.writeConfig("version",DEFAULTS.version);

SiteManager.init(NEW_INSTALLATION);

var configureGUI = new ConfigureGUI();
var searchGUI = new SearchGUI();
//--------------------------------------------------------------------------
