//	helper functions for class String

function isValid(obj){
	return obj != undefined && obj != null;
}

String.prototype.extract = function(start,end){
	var st = this;
	var startIndex = st.indexOf(start);
	if (startIndex < 0)
		return "";

	st = st.substring(startIndex + start.length);
	var endIndex = st.indexOf(end);
	if (endIndex < 0)
		return "";
	
	return st.substring(0,endIndex);
}

String.prototype.extractAll = function(start,end){
	var st = this;
	var result = [];
	while(true){
		var startIndex = st.indexOf(start);
		if (startIndex < 0)
			break;

		st = st.substring(startIndex + start.length);
		var endIndex = st.indexOf(end);
		if (endIndex < 0)
			break;
	
		result[result.length] = st.substring(0,endIndex);
		st = st.substring(endIndex + end.length);
	}
	return result;
}

String.prototype.exclude = function(start,end){
	var st = this;
	var startIndex = st.indexOf(start);
	if (startIndex < 0)
		return this;

	st = st.substring(startIndex + start.length);
	var endIndex = st.indexOf(end);
	if (endIndex < 0)
		return this;
	
	return this.substring(0,startIndex) + st.substring(endIndex+end.length);
}

String.prototype.excludeAll = function(start,end){
	var st = this;
	var temp = st;
	do{
		var l = st.length;
		temp = st.exclude(start,end);
		if (l == temp.length)
			break;
		st = temp;
	}while(true);
	return st;
}

//alert("abcdefcdsfsdceff".excludeAll("c","e"));

//+ Jonas Raoni Soares Silva
//@ http://jsfromhell.com/string/capitalize [v1.0]
/*
String.prototype.capitalize = function(){
	if (!arguments.callee.expr) {
		arguments.callee.expr = /[\w\'&]+/g;
		arguments.callee.f = function(st){
		        return st.charAt(0).toUpperCase() + st.substring(1).toLowerCase();
		};
	}
	return this.replace(arguments.callee.expr, arguments.callee.f);
};
*/

//+ Jonas Raoni Soares Silva
//@ http://jsfromhell.com/string/capitalize [rev. #2]

String.prototype.capitalize = function(){
	if (!arguments.callee.expr) {
		arguments.callee.expr = /\S+/g;
		arguments.callee.f = function(st){
		        return st.charAt(0).toUpperCase() + st.slice(1).toLowerCase();
		};
	}
	return this.replace(arguments.callee.expr, arguments.callee.f);
};

//	----------------------------------------------------------------------------------------------------------------------------------------

String.prototype.extractXmlTag = function(start){
	var tag = start.match(/<(\w+).*>/i)[1];
	return this.extract(start, "</"+tag+">");
}

String.prototype.extractAllXmlTag = function(start){
	var tag = start.match(/<(\w+).*>/i)[1];
	return this.extractAll(start, "</"+tag+">");
}

String.prototype.excludeAllXmlTag = function(start){
	var tag = start.match(/<(\w+).*>/i)[1];
	return this.excludeAll(start, "</"+tag+">");
}

String.prototype.multiReplace = function(replaces){
	var st = this;
	for(var i=0; i<replaces.length; i++){
		if (!replaces[i]) continue;
		st = st.replace(replaces[i].expr,replaces[i].str);
	}
	return st;
}

String.prototype.escapeXmlTags = function(){
	if (!arguments.callee.expr)
		arguments.callee.expr = /<[^>]*>/g;
	return this.replace(arguments.callee.expr,"");
}

String.prototype.escapeXmlChars = function(){
	if (!arguments.callee.replaces)
		arguments.callee.replaces = [
//			{expr: /&/g, str:"&amp;"},
			{expr: /\'/g, str:"&apos;"},
			{expr: /\"/g, str:"&quot;"},
			{expr: /</g, str:"&lt;"},
			{expr: />/g, str:"&gt;"},
			{expr: /&(?!amp;)(?!lt;)(?!gt;)(?!apos;)(?!quot;)(?!#)/g, str:"&amp;"},
		];
//	var st = this.replace(/&amp;/g,"&");
	return this.multiReplace(arguments.callee.replaces);
};

String.prototype.escapeHtmlEntities = function(){
	if (!arguments.callee.expr){
		arguments.callee.expr = /&(\w{2,6});/g;
		arguments.callee.f = function(entity){
			if (!arguments.callee.entities){
				arguments.callee.expr = /&(\w{2,6});/;
				arguments.callee.entities = {
					"apos":"'",
					"quot":"\"",
					"lt":"<",
					"gt":">",
					"amp":"&",
					"nbsp":" ",
					"ldquo":"“",	//	left double curly quote
					"rdquo":"”",	//	right double curly quote
					"lsquo":"‘",	//	left single curly quote
					"rsquo":"’"	//	right single curly quote
				};
			}
			entity = entity.match(arguments.callee.expr)[1];
			if (!arguments.callee.entities[entity])
				return "&"+entity+";";
			else
				return arguments.callee.entities[entity];
		};
	}
	return this.replace(arguments.callee.expr, arguments.callee.f);
};
String.prototype.escapeNumberedChars = function(){
	if (!arguments.callee.numberedCharsExp){
		arguments.callee.expr = /\&\#(\d+);*/g;
		arguments.callee.f = function(st){
			if (!arguments.callee.expr)
				arguments.callee.expr = /\&\#(\d+);*/;
			return String.fromCharCode(st.match(arguments.callee.expr)[1]);
		}
	}
	return this.replace(arguments.callee.expr,arguments.callee.f);
}

String.prototype.escapeFileNameChars = function(){
	if (!arguments.callee.expr)
		arguments.callee.expr = /[\,\;\\\/]+/mig;
	return this.replace(arguments.callee.expr, "");
}

//	http://simonwillison.net/2006/Jan/20/escape/
String.prototype.escapeRegExp = function() {
	if (!arguments.callee.expr) {
		var specials = ['/', '.', '*', '+', '?', '|','(', ')', '[', ']', '{', '}', '\\', "'", '"', /*' ',*/ '<', '>', '\n'];
		arguments.callee.expr = new RegExp('(\\' + specials.join('|\\') + ')', 'g');
	}
	return this.replace(arguments.callee.expr, '\\$1').replace(/\n/g,"n");
}

String.prototype.escapeJS = function() {
//	replace(/(\'|\")/,"\\$1");
	if (!arguments.callee.expr) {
		var specials = ["'", '"'];
		arguments.callee.expr = new RegExp('(\\' + specials.join('|\\') + ')', 'g');
	}
	return this.replace(arguments.callee.expr, '\\$1').replace(/\n/g,"n");
}
//alert("[a]sdfs[/a]".extract("[a]","[/a]"));
//alert("dsf<div class='lyricbox' >abcd\n1234\ndcba<p>dsf".extract("<div class='lyricbox' >","<p>"));

//alert("abc'd\"e[](){}".escapeRegExp());

//	----------------------------------------------------------------------------------------------------------------------------------------

function globalMatchFound(st){
	if (st==null)
		return false;
	return st.length>0;
}

String.prototype.fillParams = function(params){
	if (!arguments.callee.expr){
		arguments.callee.expr = /\{(\w{2,10}):([\w\d )]{2,20}\})/g;
		arguments.callee.f = function(st, params){
			if (!arguments.callee.expr)
				arguments.callee.expr = /\{(\w{2,10}):([\w\d\ )]{2,20})\}/;
			st = st.match(arguments.callee.expr);
			if (!params[st[1]])
				return st[1] + ":" + st[2];
//			alert(st + "\n" + params[st[1]][st[2]]);
			return params[st[1]][st[2]];
		}
	}
	var callF = arguments.callee.f;
	var f = function(st){
		return callF(st, params);
	}
	var st = this;
	while(globalMatchFound(st.match(arguments.callee.expr)))
//	while(st.match(arguments.callee.expr) != null)
		st = st.replace(arguments.callee.expr,f);

	return st;
}

String.prototype.fill1 = function(params){
	if (!arguments.callee.expr){
		arguments.callee.expr = /\{([\w\d]{1,20})\}/g;
		arguments.callee.f = function(st, params){
			if (!arguments.callee.expr)
				arguments.callee.expr = /\{([\w\d]{1,20})\}/;
			st = st.match(arguments.callee.expr);
//			alert(st + "\n" + params[st[1]][st[2]]);
			if (!params[st[1]])
				return "{"+st[1]+"}";
			return params[st[1]];
		}
	}
	var callF = arguments.callee.f;
	var f = function(st){
		return callF(st, params);
	}
	var st = this;
//	while(st.match(arguments.callee.expr).length>0)
		st = st.replace(arguments.callee.expr,f);

	return st;
}

function binToInt(){
	var n = 0;
	for(var i=0; i<arguments.length; i++)
		n = n*2 + arguments[i];
	return n;
}

//	----------------------------------------------------------------------------------------------------------------------------------------

/*
function typeOf(value){
	var type = typeof(value);
	return type;
	if (type != "object")
		return type;
	if (Object.prototype.toString.apply(value) === '[object Array]')
//	if (value.length != undefined)
		return "array";
	return type;
}
*/

//alert(eval("("+JSON.stringify({a:["lyrics"]})+")"));

/*
alert(JSON.stringify({
	sf:"sdfsdf",
	fdgd:346
}));
*/

