//----- settings dialog ------------------------------------------------------------

function WindowController(context, file, menu, item, icon, title){
	this.context = context;
	this.file = file;
	this.menu = menu;
	this.item = item;
	this.icon = icon;
	this.title = title;
	this.window = undefined;
	this.registerButton();
}

WindowController.prototype.qtBindingLoaded = false;

WindowController.prototype.loadFromFile = function(){
	if (this.window)
		return false;
	var UIloader = new QUiLoader( this );
	var uifile = new QFile ( Amarok.Info.scriptPath() + "/" + this.file );
	uifile.open( QIODevice.ReadOnly );
	this.window = UIloader.load( uifile, this); //load the ui file
	uifile.close();
	return true;
}

WindowController.prototype.loadQtBinding = function(){
	if (WindowController.prototype.qtBindingLoaded)
		return false;
	Importer.loadQtBinding( "qt.gui");
	Importer.loadQtBinding( "qt.uitools");
	WindowController.prototype.qtBindingLoaded = true;
	return true;
}

WindowController.prototype.open = function(){
	try{
		this.loadQtBinding();
		if(this.loadFromFile())
			this.context.onLoad(this);
		this.context.onOpen(this);
		this.window.show();
	}catch(e){
		alert(e);
	}
}

WindowController.prototype.registerButton = function(){
	Amarok.Window["add" + this.menu]( this.item, this.title, this.icon ); //: add the configuration menu
	Amarok.Window[this.menu][this.item]['triggered()'].connect(this, this.open) //add a signal
}
/*
ConfigureGUI.prototype.load = function(){
	this.loadQtBinding();
	if (!this.mainWindow || this.mainWindow == undefined){
		this.mainWindow = this.loadFile("configure.ui");

		this.status = new QStatusBar();
		this.mainWindow.setStatusBar(this.status);
		this.bind();
	}
	this.init();
}

ConfigureGUI.prototype.loadSearch = function(){
	this.loadQtBinding();
	if (!this.mainWindow || this.mainWindow == undefined){
		this.searchWindow = this.loadFile("search.ui");
		this.bindSearch();
	}
}

ConfigureGUI.prototype.registerButton = function(){
	Amarok.Window.addSettingsMenu( "settings", qsTranslate("SettingsGUI","Ultimate Lyrics.."),"configure" ); //: add the configuration menu
	Amarok.Window.SettingsMenu.settings['triggered()'].connect(this, function(){
		this.load();
		this.status.showMessage('');
		this.mainWindow.show();
	}) //add a signal
	Amarok.Window.addToolsMenu( "tools", qsTranslate("SettingsGUI","Ultimate Lyrics Search"),"search" ); //: add the configuration menu
	Amarok.Window.ToolsMenu.tools['triggered()'].connect(this, function(){
		this.loadSearch();
		this.searchWindow.show();
		this.resetSearch();
	}) //add a signal
//	this.mainWindow.show();
}
*/
function SearchGUI(){
	var toolsWindow = new WindowController(this, "search.ui", "ToolsMenu", "ul_tools", "edit-find"/*"document-edit-verify"*/, qsTranslate("SearchGUI","Ultimate Lyrics Search"));
}

SearchGUI.prototype.onLoad = function(windowController){
	this.window = windowController.window;

	with(this.window){
		this.labelStatus = new QLabel("");
		statusbar.addPermanentWidget(this.labelStatus);
	}

	with(this.window.centralwidget){
		buttonBox.button(QDialogButtonBox.Retry).clicked.connect(this, function(){
			try{
				this.reloadLyrics();
			}catch(e){
				alert(e);
			}
		})
		buttonBox.button(QDialogButtonBox.Reset).clicked.connect(this, function(){
			try{
				this.reset(true);
			}catch(e){
				alert(e);
			}
		})
		buttonBox.button(QDialogButtonBox.RestoreDefaults).clicked.connect(this, function(){
			try{
				this.reset(false);
			}catch(e){
				alert(e);
			}
		})
	}

	Amarok.Engine.trackChanged.connect(
		function() {
			searchGUI.reset(true);
	//		alert(artist + " | " + title);
		}
	)
}

SearchGUI.prototype.onOpen = function(windowController){
	this.reset(true);
}

SearchGUI.prototype.reset = function(format){
	var artist = Amarok.Engine.currentTrack().artist;
	var title = Amarok.Engine.currentTrack().title;

	if (format){
		var info = new SearchInfo(artist, title, "");
		artist = info.artist;
		title = info.title;
	}

	this.changeSite(false);
	this.update(artist, title);
}

SearchGUI.prototype.update = function(artist, title){
	if (!this.window)
		return;
//	this.info = info;
	with(this.window.centralwidget){
		lineEditArtist.text = artist;
		lineEditTitle.text = title;
	}
}

SearchGUI.prototype.reloadLyrics = function(){
	if (!this.window)
		return;
	with(this.window.centralwidget){
		var artist = lineEditArtist.text;
		var title = lineEditTitle.text;
		var info = new SearchInfo(artist, title, "");
		info.artist = artist;
		info.title = title;
//		alert(info);
		SiteManager.fetchLyrics(info);
	}
}

SearchGUI.prototype.changeSite = function(displaySite){
	try{
		if (!isValid(this.window))
			return;
		with(this.window){
			this.labelStatus.setText(SiteManager.working ? qsTranslate("SettingsGUI","searching") : qsTranslate("SettingsGUI","idle"));
			if (isValid(SiteManager.currentSite))
				statusbar.showMessage(displaySite ? SiteManager.currentSite.domain : " ");
		}
	}catch(e){
		alert(e);
	}
}

function ConfigureGUI(){
	var settingsWindow = new WindowController(this, "configure.ui", "SettingsMenu", "ul_settings", "configure", qsTranslate("SettingsGUI","Ultimate Lyrics..."));
}

ConfigureGUI.prototype.onLoad = function(windowController){
	if (windowController)
		this.mainWindow = windowController.window;
	with(this.mainWindow){
//			change.accepted.connect( function(){gui.save();gui.mainWindow.hide()} );
//			change.rejected.connect( function(){gui.mainWindow.hide()} );
		change.button(QDialogButtonBox.Apply).clicked.connect(this, function(){
			try{
				var saved = this.save();
			}catch(e){
				alert(e);
			}
		})
		change.button(QDialogButtonBox.Ok).clicked.connect(this, function(){
			try{
				var saved = this.save();
				if (saved)
					this.mainWindow.hide();
			}catch(e){
				alert(e);
			}
		})
		change.button(QDialogButtonBox.Cancel).clicked.connect(this, function(){
			try{
				this.mainWindow.hide();
			}catch(e){
				alert(e);
			}
		})
		change.button(QDialogButtonBox.RestoreDefaults).clicked.connect(this, function(){
			try{
				settings = config.restoreDefaults();
				SiteManager.init();
				this.onLoad();
				this.init();
				this.showMessage(qsTranslate("SettingsGUI",'default settings restored'));
			}catch(e){
				alert(e);
			}
		})
		with(tabWidget.children()[0]){
			with(tabDisplay.groupLimit){
//				displayMode.clear();
//				displayMode.addItems(this.getLabelValues("display_mode"));
				htmlLimit.clear();
				lyricsLimit.clear();
				htmlLimit.addItems(this.getLabelValues("max_size_html"));
				lyricsLimit.addItems(this.getLabelValues("max_size_lyrics"));
			}
			with(tabGoogleSearch.groupGoogleSearch){
				googleNo.clear();
				googleNo.addItems(this.getLabelValues("gsearch_no"));
				var googleSuffixes = [];
				var sites = settings.sites;
				for(var i=0;i<settings.gsearch_domains.length;i++)
					googleSuffixes[googleSuffixes.length] = settings.gsearch_domains[i];
				for(var i in sites)
					if (sites[i].gSearchable)
						googleSuffixes[googleSuffixes.length] = SiteManager.extractDomain(i);
//				googleSuffix.clear();
//				googleSuffix.addItems(googleSuffixes);
				while(googleSuffix.takeItem(0));
				googleSuffix.insertItems(0,googleSuffixes);
			}
/*
			with(tabGoogleSearch.groupGoogleFilter){
				googleFilterURL.clear();
				googleFilterURL.addItems(this.getLabelValues("gsearch_filter_url"));
			}
*/
			with(tabCache.fileCache){
				clearCache.clicked.connect(this, function(){this.showMessage(SiteManager.fileCache.clear()?qsTranslate("SettingsGUI",'file cache cleared'):qsTranslate("SettingsGUI",'file cache not cleared'));});
				clearCacheExpired.clicked.connect(this, function(){this.showMessage(SiteManager.fileCache.clearExpired(true)?qsTranslate("SettingsGUI",'expired files removed'):qsTranslate("SettingsGUI",'some expired files are not removed'));});
				fileCacheExpires.clear();
				fileCacheExpires.addItems(this.getLabelValues("file_cache_expires"));
			}
			with(tabSites){
				headerView = tableWidget.horizontalHeader();
				headerView.setResizeMode(QHeaderView.Stretch);
				headerView.setResizeMode(1, QHeaderView.Interactive);
//					headerView.setResizeMode(2, QHeaderView.Interactive);
				tableWidget.setColumnWidth(1,100);
//					tableWidget.setColumnWidth(2,100);
//				tableWidget.setStyleSheet("QTableWidget::item {border-width: 1px; border-style: groove; padding : 3px}");
//				tableWidget.setStyleSheet("QTableWidget::item {border-width: 3px}");

				tableWidget.move = function(displace){
					var sourceRow = this.currentRow();
					var destRow = sourceRow+displace;
					if (sourceRow < 0 || sourceRow >= this.rowCount || destRow < 0 || destRow >= this.rowCount)
						return;
					// take whole rows
					var sourceItems = this.takeRow(sourceRow);
					var destItems = this.takeRow(destRow);
//						alert(sourceItems + "   " + destItems);
 
					// set back in reverse order
					this.setRow(sourceRow, destItems);
					this.setRow(destRow, sourceItems);
					this.setCurrentCell(destRow,0);
				}

				// takes and returns the whole row
				tableWidget.takeRow = function(row){
					var cb1 = new QCheckBox();
					cb1.setCheckState(this.cellWidget(row,1).checkState());
					cb1.setEnabled(this.cellWidget(row,1).enabled);
/*
					var cb2 = new QCheckBox();
					cb2.setCheckState(this.cellWidget(row,2).checkState());
					cb2.setEnabled(this.cellWidget(row,2).enabled);
*/
					var label = new QLabel(this.cellWidget(row,0).text);
					label.setStyleSheet("QLabel {padding-left : 15px}");
					var rowItems=[
						label,
						cb1,
//							cb2,
					];

					return rowItems;
				}

				// sets the whole row
				tableWidget.setRow = function(row, rowItems){
					for (var col = 0; col < this.columnCount; col++)
						this.setCellWidget(row, col, rowItems[col]);
				}
//				sitePriorityUp
				commandLinkButtonSiteUp.clicked.connect(tableWidget, function(){
					try{
						this.move(-1);
					}catch(e){
//						alert(e);
					}
				})
//				sitePriorityDown
				commandLinkButtonSiteDown.clicked.connect(tableWidget, function(){
					try{
						this.move(+1);
					}catch(e){
//						alert(e);
					}
				})
			}
		}
	}
}

ConfigureGUI.prototype.onOpen = function(windowController){
	this.showMessage('');
	this.init();
}

ConfigureGUI.prototype.showMessage = function(msg){
	with(this.mainWindow){
		statusbar.showMessage(msg);
	}
}

ConfigureGUI.prototype.init = function(){
	with(this.mainWindow.tabWidget.children()[0]){
		with(tabDisplay){
			this.initRadioButtons(groupMode,"radioButtonDisplayMode","display_mode");
		}
		with(tabDisplay.groupRotate){
			rotatingLyrics.checked = settings.rotating_mode;
		}
		with(tabDisplay.groupShow){
			displayTitle.checked = settings.display_title;
			displayUrl.checked = settings.display_url;
//			displayMode.setCurrentIndex(displayMode.findText(this.getLabelValue("display_mode")));
		}
		with(tabDisplay.groupWikia){
			multiLangWikia.checked = settings.multi_lang_wikia;
		}
		with(tabDisplay.groupLimit){
			htmlLimit.setCurrentIndex(htmlLimit.findText(this.getLabelValue("max_size_html")));
			lyricsLimit.setCurrentIndex(lyricsLimit.findText(this.getLabelValue("max_size_lyrics")));
		}
		with(tabSearch.groupAdjustment){
			omitParenthesises.checked = settings.omit_parenthesises;
			omitBrackets.checked = settings.omit_brackets;
			omitThe.checked = settings.omit_the;
			omitTrackNumber.checked = settings.omit_track_number;
			replaceAmp.checked = settings.replace_amp;
		}
		with(tabGoogleSearch.groupGoogleSearch){
			googleNo.setCurrentIndex(googleNo.findText(settings.gsearch_no));
//			googleSuffix.setCurrentIndex(googleSuffix.findText(settings.gsearch));
			googleSuffix.setCurrentItem(googleSuffix.findItems(settings.gsearch,Qt.MatchExactly)[0]);
		}
		with(tabGoogleSearch)
			this.initRadioButtons(groupGoogleFilter,"radioButtonGoogleFilterURL","gsearch_filter_url");
		with(tabGoogleSearch.groupGoogleFilter){
			googleFilterArtist.checked = settings.gsearch_filter_artist;
//			googleFilterURL.setCurrentIndex(googleFilterURL.findText(this.getLabelValue("gsearch_filter_url")));
			googleFilterCriteria.text = settings.gsearch_filter_expr;
		}
		with(tabCache){
/*
			with(cacheOptions){
				noCacheOption.checked = settings.cache == "no cache";
				fileCacheOption.checked = settings.cache == "file cache";
			}
*/
			with(fileCache){
				checked = settings.cache == "file cache";
				fileCacheExpires.setCurrentIndex(fileCacheExpires.findText(this.getLabelValue("file_cache_expires")));
				fileCacheAutoRemove.setCheckState(settings.file_cache_auto_remove?Qt.Checked:Qt.Unchecked);
			}
		}
		with(tabSites){
			var row = 0;
			for (var siteName in settings.sites){
				var cb1 = new QCheckBox();
				var label = new QLabel(siteName);
				label.setStyleSheet("QLabel {padding-left : 5px}");

				cb1.setCheckState(settings.sites[siteName]['search']?Qt.Checked:Qt.Unchecked);
				cb1.setEnabled(!settings.sites[siteName].searchLock);
/*
				var cb2 = new QCheckBox();
				cb2.setCheckState(settings.sites[siteName]['cache']?Qt.Checked:Qt.Unchecked);
				cb2.setEnabled(!settings.sites[siteName].cacheLock);
*/
				tableWidget.setCellWidget(row,0,label);
				tableWidget.setCellWidget(row,1,cb1);
//				tableWidget.setCellWidget(row,2,cb2);
				row++;
			}
			for(;row<tableWidget.rowCount;row++){
				tableWidget.removeCellWidget(row,0);
				tableWidget.removeCellWidget(row,1);
//				tableWidget.removeCellWidget(row,2);
			}
		}
	}
}

ConfigureGUI.prototype.save = function(){
	if (SiteManager.working){
		this.showMessage(qsTranslate("SettingsGUI","Ultimate Lyrics are now searching, and changes can't be saved"));
		return false;
	}
	with(this.mainWindow.tabWidget.children()[0]){
		with(tabDisplay.groupRotate){
			settings.rotating_mode = rotatingLyrics.checked;
		}
		with(tabDisplay.groupShow){
			settings.display_title = displayTitle.checked;
			settings.display_url = displayUrl.checked;
		}
		with(tabDisplay.groupWikia){
			settings.multi_lang_wikia = multiLangWikia.checked;
		}
		with(tabDisplay.groupLimit){
//			this.setLabelValue("display_mode", displayMode.currentText);
			this.setLabelValue("max_size_html", htmlLimit.currentText);
			this.setLabelValue("max_size_lyrics", lyricsLimit.currentText);
		}
		with(tabDisplay){
			settings.display_mode = this.getRadioButtons(groupMode,"radioButtonDisplayMode","display_mode");
		}
		with(tabSearch.groupAdjustment){
			settings.omit_the = omitThe.checked;
			settings.omit_track_number = omitTrackNumber.checked;
			settings.replace_amp = replaceAmp.checked;
		}
		with(tabGoogleSearch.groupGoogleSearch){
			this.setLabelValue("gsearch_no", googleNo.currentText);
//			settings.gsearch = googleSuffix.currentText;
			settings.gsearch = googleSuffix.item(googleSuffix.currentRow).text();

		}
		with(tabGoogleSearch.groupGoogleFilter){
			settings.gsearch_filter_artist = googleFilterArtist.checked;
//			this.setLabelValue("gsearch_filter_url", googleFilterURL.currentText);
			settings.gsearch_filter_expr = googleFilterCriteria.text;
		}
		with(tabGoogleSearch)
			settings.gsearch_filter_url = this.getRadioButtons(groupGoogleFilter,"radioButtonGoogleFilterURL","gsearch_filter_url");
		with(tabCache){
/*
			with(cacheOptions){
				if (noCacheOption.checked)
					settings.cache = "no cache";
				if (fileCacheOption.checked)
					settings.cache = "file cache";
			}
*/
			with(fileCache){
				settings.cache = checked? "file cache" : "no cache";
				this.setLabelValue("file_cache_expires",fileCacheExpires.currentText);

				settings.file_cache_auto_remove = fileCacheAutoRemove.checked;//State()===Qt.Checked;
			}

		}
		with(tabSites){
			setEnabled(false);
			var row = 0;
			var sites = {};
			for (var row=0; row< tableWidget.rowCount; row++){
				if (tableWidget.cellWidget(row,1)==undefined)
					continue;
				var cb1 = tableWidget.cellWidget(row,1).checkState()===Qt.Checked;
//				var cb2 = tableWidget.cellWidget(row,2).checkState()===Qt.Checked;
				var siteName = tableWidget.cellWidget(row,0).text;

				sites[siteName] = settings.sites[siteName];
//				alert('siteName ' + siteName);
				sites[siteName]["search"] = cb1;
//				sites[siteName]["cache"] = cb2;
			}
			setEnabled(true); //	no idea why is this necessary
//			settings.set("sites",sites);
			settings.sites = sites;
		}
	}
	config.save(settings);
	SiteManager.init();
	this.showMessage(qsTranslate("SettingsGUI",'changes applied'));
	return true;
}

ConfigureGUI.prototype.labels = {
	display_mode:[
		["Xml",qsTranslate("SettingsGUI","Plain Text")],
		["Html",qsTranslate("SettingsGUI","Rich Text")]
	],
	max_size_lyrics:[
		[0.2,"200 "+qsTranslate("SettingsGUI","characters")],
		[4,"4,000 "+qsTranslate("SettingsGUI","characters")],
		[8,"8,000 "+qsTranslate("SettingsGUI","characters")],
		[16,"16,000 "+qsTranslate("SettingsGUI","characters")],
		[64,"64,000 "+qsTranslate("SettingsGUI","characters")],
		[1024,"1,000,000 "+qsTranslate("SettingsGUI","characters")],
	],
	max_size_html:[
		[4,"4 KB"],
		[128,"128 KB"],
		[256,"256 KB"],
		[512,"512 KB"],
//		[1024,"1 MB"],
	],
	gsearch_no:[
		[5,5],
		[10,10],
		[15,15],
		[20,20]
	],
	gsearch_filter_url:[
		["no filter",qsTranslate("SettingsGUI","Do not Apply (permissive)")],
		["full url",qsTranslate("SettingsGUI","Apply to Full URL (less restrictive)")],
		["hostname",qsTranslate("SettingsGUI","Apply to Hostname (more restrictive)")]
	],
	file_cache_expires:[
//		[1*60*1000,"one minute"],
//		[5*60*1000,"five minutes"],
		[30*60*1000,qsTranslate("SettingsGUI","30 minutes")],
		[6*60*60*1000,qsTranslate("SettingsGUI","6 hours")],
		[1*24*60*60*1000,qsTranslate("SettingsGUI","one day")],
		[7*24*60*60*1000,qsTranslate("SettingsGUI","7 days")],
		[30*24*60*60*1000,qsTranslate("SettingsGUI","one month")],
		[30*12*24*60*60*1000,qsTranslate("SettingsGUI","one year")]
	]
}

ConfigureGUI.prototype.getLabel = function(key,keyColumn,value){
	if (value == undefined)
		value = settings[key];
	var valueColumn = (keyColumn+1)%2;
	var labels = this.labels[key];
	for(var i=0; i<labels.length; i++)
		if(labels[i][keyColumn]==value)
			return labels[i][valueColumn];
}

ConfigureGUI.prototype.getLabelValue = function(key){
	return this.getLabel(key,0);
}

ConfigureGUI.prototype.setLabelValue = function(key,value){
	return settings[key] = this.getLabel(key,1,value);
}

ConfigureGUI.prototype.getLabelKeys = function(key){
	var labels = this.labels[key];
	var values = [];
	for(var i=0; i<labels.length; i++)
		values[values.length] = labels[i][0];
	return values;
}

ConfigureGUI.prototype.getLabelValues = function(key){
	var labels = this.labels[key];
	var values = [];
	for(var i=0; i<labels.length; i++)
		values[values.length] = labels[i][1];
	return values;
}

ConfigureGUI.prototype.initRadioButtons = function(container, prefix, key){
	var i=1;
	while(true){
		var button = container[prefix+i];
//		Amarok.alert(container + " " + prefix + " " + i + "" + button);
		if (!button)
			break;
//		Amarok.alert(this.getLabelValue(key) +  " " + button.text);
		button.checked = this.getLabelValue(key) == button.text;
		i++;
	}
}

ConfigureGUI.prototype.getRadioButtons = function(container, prefix, key){
	var i=1;
	while(true){
		var button = container[prefix+i];
//		Amarok.alert(container + " " + prefix + " " + i + "" + button);
		if (!button)
			break;
		if (button.checked){
//			alert(button.text);
			return this.getLabel(key,1,button.text.replace(/&/g,""));
		}
//		Amarok.alert(this.getLabelValue(key) +  " " + button.text);
//		button.checked = this.getLabelValue(key) == ;
		i++;
	}
}

//    mainWindow.children()[0].accepted.connect( saveConfiguration ); //when OK button is clicked
//    mainWindow.children()[0].rejected.connect( readConfiguration ); //call the function when Cancel button is clicked

