//	--------------------------------------------------------------------------------------------------------------------
//	base class for all classes that represent actual site
function LyricsClass(){
	this.charset = "iso-8859-1";
	this.useCache = true;
	this.display_mode = settings.display_mode;
}

LyricsClass.prototype.multiple = false;

LyricsClass.prototype.getUrl = function(info){
	// no calculating url or url is already known
	if (info.url!="" || !this.url)
		return info.url;

	var album = info.track.album.multiReplace(SearchInfo.prototype.defaultReplaces);
	var Title = info.arrange(info.title,this.urlFormat,false);
	var Title2 = Title;
	Title2 = Title.toLowerCase();
	Title2 = Title2.capitalize();
	var params = {
		artist:	info.arrange(info.artist,this.urlFormat,true),
		title:	info.arrange(info.title,this.urlFormat,true),
		album:	info.arrange(album,this.urlFormat,true),
		Artist:	info.arrange(info.artist,this.urlFormat,false),
		Title:	info.arrange(info.title,this.urlFormat,false),
		Title2:	Title2,
		Album:	info.arrange(album,this.urlFormat,false)
	}
	params.a = params.artist.charAt(0);

	return this.url.fill1(params);
}

LyricsClass.prototype.join = function(obj){
	for(var i in obj)
		this[i] = obj[i];
}

LyricsClass.prototype.filter = function(info){
	return info.isValid;
}

LyricsClass.prototype.setup = function(){}

LyricsClass.prototype.init = function(charset){
	if (charset != undefined)
		this.charset = charset;
	this.charset = this.charset.toUpperCase();
	var codec = QTextCodec.codecForName(new QByteArray(this.charset));
	if (codec != undefined)
		this.codec = codec;
	if (this.urlFormat)
	for(var i=0;i<this.urlFormat.length;i++)
		if (this.urlFormat[i].punct){
			this.urlFormat[i].expr = new RegExp("[" + this.urlFormat[i].punct.escapeRegExp() + "]+","g");
			this.urlFormat[i].str = this.urlFormat[i].rep;
//			alert(this.urlFormat[i].expr);
		}
	if (this.title){
		var mask = this.title.escapeRegExp();
		this.titleOrder = mask.indexOf("\\{artist\\}") < mask.indexOf("\\{title\\}");
		mask = mask.replace(/\\\{[^\\]*\\\}/g,"(.*)");
		this.titleMask = new RegExp(mask,"i");
	}
}

LyricsClass.prototype.fetchReply = function(handler,info){
//	alert(this.domain + "|" + info.searchUrl);
//	Amarok.alert("http://translate.google.com/translate?hl=en&sl=en&tl=sr&u=" + new QUrl(info.searchUrl));
	new DataDownloader( new QUrl(info.searchUrl), handler);
}

LyricsClass.prototype.downloadableReply = function(info){
	return info.searchUrl.indexOf(this.domain)>=0 && ((info.url=="" && !this.skipSearch) || info.url!="");
}

LyricsClass.prototype.processRequest = function(handler,info){
	info.timestamp1 = new Date();
	info.searchUrl = this.getUrl(info);
	if (this.filter(info) && this.downloadableReply(info) && (!this.cache.isValid(info,this) && this.useCache || !this.useCache))
		this.fetchReply(handler,info)
	else
		handler("");
}

LyricsClass.prototype.processReply = function(reply,info){
	info.timestamp2 = new Date();
	if (!this.filter(info) || !this.downloadableReply(info))
		return false;

	reply = this.decodeReply(reply);
//	alert(reply);
	this.matchInfo(reply, info);
	reply = this.createReply(reply,info);
	reply = this.cache.read(info,this,reply);

	if(reply=="" || SiteManager.pendingRequest){
//		alert("empty lyrics: " + this.domain + " | " + info.searchUrl);
		return false;
	}

	this.showReply(reply,info);
	return true;
}

LyricsClass.prototype.decodeReply = function(reply){
	if (!arguments.callee.charsetExp)
		arguments.callee.charsetExp = /charset=([\w\d-]*)\"/mi;
	var decodedReply = this.codec.toUnicode(reply);
	var charset = decodedReply.match(arguments.callee.charsetExp);
	charset = charset!=null ? charset[1] : "";
	charset = charset.toUpperCase();
	if (charset != "" && charset != this.charset){
		alert("charset for " + this.site + " has been changed from " + this.charset + " to " + charset);
		this.init(charset);
		return this.decodeReply(reply);
	}
	return decodedReply;
}

LyricsClass.prototype.matchInfo = function(reply,info){
	if (!arguments.callee.re)
		arguments.callee.re = /<title>(.*)<\/title>/i;
		arguments.callee.titleReplaces = [
			{expr:/lyrics/gi, str:" "},
			{expr:/::/g, str:" "},
			{expr:/ *[-|] /g, str:" "},
			{expr:/ +/g, str:" "},
			{expr:/[\r\t\n]/g, str:""}
		];

	var title = reply.match(arguments.callee.re);
	var artist = "";
	if (title != null)
		title = title[1].replace(/[\r\t\n]/g,"");
	else{
		title = info.title;
		artist = info.artist;
	}

	if (this.titleMask){
		var matchInfo = title.match(this.titleMask);
		if (matchInfo != null){
			title = matchInfo[1+this.titleOrder];
			artist = matchInfo[2-this.titleOrder];
		}
	}else{
		var re = new RegExp("(" + info.artist.escapeRegExp() + ")","i");
		var matchInfo = title.match(re);
/*
		if (matchInfo != null){
//			title = title.replace(new RegExp(" *[-|]?(by)?(::)? *" + info.artist + " *[-|]? *","gi"), "").replace(/lyrics/gi, "");
			title = title.replace(new RegExp(" *(by)? *" + info.artist,"gi"), " ");
			artist = matchInfo[1];
		}
*/
		title = title.multiReplace(arguments.callee.titleReplaces)
	}

	info.matchedTitle = title;
	info.matchedArtist = artist;
}

LyricsClass.prototype.getReply = function(html){
	var reply = html;

	if (!arguments.callee.extract){
		arguments.callee.extract = function(reply, rules, multiple){
			if (!multiple)
				if (typeof(rules)=="string")
					reply = reply.extractXmlTag(rules);
				else
					reply = reply.extract(rules[0],rules[1]);
			else
				if (typeof(rules)=="string")
					reply = reply.extractAllXmlTag(rules);
				else{
					reply = reply.extractAll(rules[0],rules[1]);
					reply = reply.join("\n\n<hr/>\n");
				}
			return reply;
		}
		arguments.callee.extractAll = function(reply, rules){
			return reply;
		}
		arguments.callee.exclude = function(reply, rules){
			if (typeof(rules)=="string")
				reply = reply.excludeAllXmlTag(rules);
			else
				reply = reply.excludeAll(rules[0],rules[1]);
			return reply;
		}
	}

	if (this.invalidIndicator){
	 	if (typeof(this.invalidIndicator)=="string"){
			if (reply.indexOf(this.invalidIndicator)>0)
				return "";
		}else for(var i=0; i<this.invalidIndicator.length; i++){
			if (reply.indexOf(this.invalidIndicator[i])>0)
				return "";
		}
	}

	for(var i=1; i<5; i++){
		var rules = this["extract"+ (i==1 ? "" : i)];
		if (!rules)
			return "";
		var temp = reply;
//		alert(rules + "\n" + temp);
		if (typeof(rules)=="string")
			temp = arguments.callee.extract(temp, rules, this.multiple);
		else for(var j=0; j<rules.length; j++)
			temp = arguments.callee.extract(temp, rules[j], this.multiple);

//		alert(temp);
		if (temp != ""){
			reply = temp;
			break;
		}
	}

	var rules = this.exclude;
	if (rules){
		if (typeof(rules)=="string")
			reply = arguments.callee.exclude(reply, rules);
		else for(var j=0; j<rules.length; j++)
			reply = arguments.callee.exclude(reply, rules[j]);
	}

	return reply;
}

LyricsClass.prototype.postExtractReplaces = [
	{expr:/[\r\t]/g, str:""},				//	removes return chars
	{expr:/ +/g, str:" "},					//	compress multiple spaces
	{expr:/ *<br[ \n]*\/?> *\n* */mig, str:"<br/>\n"},	//	adds \n after <br/> when missing
	{expr:/^[ \n]*(<br\/>\n)*/m, str:""},			//	removes leading \n and <br/>
	{expr:/(<br\/>\n)*[ \n]*$/m, str:""},			//	removes ending <br/> and \n
	{expr:/(<br\/>\n){2,}/mg, str:"<br/>\n<br/>\n"},	//	compress multiple <br/>
];

LyricsClass.prototype.createReply = function(reply,info){
	info.fetchedSize = Math.ceil(reply.length/1024);
	if (info.fetchedSize > settings.max_size_html)
		reply = "<i>{msg:too big web page}</i><br/>\n<br/>\n{reply:fetch_info}";
	else{
		reply = this.getReply(reply,info);
		reply = reply.escapeNumberedChars();
		reply = reply.multiReplace(this.postExtractReplaces);
	}

	info.extractedSize = Math.ceil(reply.length/1024);
	if (info.extractedSize > settings.max_size_lyrics)
		reply = "<i>{msg:too long lyrics}</i><br/>\n<br/>\n{reply:fetch_info}";

	if (reply != "")
		reply = "<title>"+info.matchedTitle.capitalize() + (info.matchedArtist != "" ? " - ":"") + info.matchedArtist.capitalize() +"</title>\n" + reply;

	return reply;
}

LyricsClass.prototype.showReply = function(reply, info){
	if (!arguments.callee.templates){
		arguments.callee.templates = {
			Suggestions:{
				page:'<suggestions page_url="{reply:url}">\n{reply:reply}\n</suggestions>'
			},
			Xml:{
				page:'<?xml version="1.0" encoding="{lyrics:charset}" ?><lyric artist="{replyInfo:artist}" title="{replyInfo:title}" page_url="{reply:url}">{reply:fetch_title}{reply:reply}{reply:fetch_url}{reply:debug}</lyric>',
				//:	footer info about fetching
				debug:'\n\nhandler: {lyrics:domain}\n{reply:url}\nfetched in: {reply:fetchedTime} s\n{template:fetch_info}',
				fetch_info:qsTranslate("lyrics","fetched from web")+": {info:fetchedSize} KB ("+qsTranslate("lyrics","limit")+": {settings:max_size_html} KB)\n"+qsTranslate("lyrics","extracted")+": {info:extractedSize} KB ("+qsTranslate("lyrics","limit")+": {settings:max_size_lyrics} KB)",
//"{reply:title}"  
				fetch_title:'"{reply:host}"\n\n',
				fetch_url:'\n\n{reply:url}'
			},
			Html:{
				//	<style type="text/css">body{color: purple; background-color: silver} a:link{color: GhostWhite; text-decoration: none}</style>
				page:'<html><head><title>{reply:title}</title><meta http-equiv="Content-Type" content="text/html; charset={lyrics:charset}"></head><body>{reply:fetch_title}<div>{reply:reply}</div>{reply:fetch_url}{reply:debug}</body></html>',
				//:	footer info about fetching
				debug:'<hr/>handler: {lyrics:domain}<br/><a href="{reply:url}">{reply:url}</a><br/>fetched in: {reply:fetchedTime} s<br/>{template:fetch_info}',
				fetch_info:qsTranslate("lyrics","fetched from web")+": {info:fetchedSize} KB ("+qsTranslate("lyrics","limit")+": {settings:max_size_html} KB)<br/>"+qsTranslate("lyrics","extracted")+": {info:extractedSize} KB ("+qsTranslate("lyrics","limit")+": {settings:max_size_lyrics} KB)",
//<strong>{reply:title}</strong><br/>
				fetch_title:'<center><a href="{reply:url}" title="{reply:url}"><em>{reply:host}</em></a></center><hr/>',
				fetch_url:'<hr/><center>{reply:url}</center>'
			}
		}
		arguments.callee.messages = {
			"too big web page" : qsTranslate("lyrics","Ultimate Lyrics are skipping the page.") + " " + qsTranslate("lyrics","The HTML page is too big for analizing. It could freeze the player for some time."),
			"too long lyrics" : qsTranslate("lyrics","Ultimate Lyrics are skipping the page.")+ " " + qsTranslate("lyrics","Text extracted from the HTML page is too long. The script conceives the page does not contain lyrics."),
			"no text found" : qsTranslate("lyrics","Ultimate Lyrics are skipping the page.")+ " " + qsTranslate("lyrics","No text found.")
		}
	}

	var title = reply.match(/<title>(.*)<\/title>\n/i);
	if (title != null)
		title = title[1];

	reply = reply.replace(/<title>.*<\/title>\n/i,"");

	var replyInfo = title.match(/(.*) - (.*)/i);
	replyInfo = (replyInfo != null) ? {title: replyInfo[1], artist: replyInfo[2]} : {title: info.title, artist: info.artist};

	if (this.display_mode == "Xml"){
		reply = reply.escapeHtmlEntities();
		reply = reply.escapeXmlTags();
		reply = reply.escapeXmlChars();
	}

	var url = (info.searchUrl).escapeXmlChars();
	var template = arguments.callee.templates[this.display_mode];
	var params = {
		info:info,
		lyrics:this,
		settings:settings,
		template:template,
		replyInfo:replyInfo,
		reply:{
			title:title.escapeXmlChars(),
			reply:reply,
			url:url,
			host:(new QUrl(url)).host(),
			fetchedTime:info.getFetchedTime(),
			debug:DEBUG?template.debug:"",
			fetch_title:settings.display_title?template.fetch_title:"",
			fetch_url:settings.display_url?template.fetch_url:"",
			fetch_info:template.fetch_info
		},
		msg:arguments.callee.messages
	}
	var page = template.page.fillParams(params);
	if (this.display_mode == "Xml" || this.display_mode == "Suggestions")
		Amarok.Lyrics.showLyrics(page);
	else
		Amarok.Lyrics.showLyricsHtml(page);
//	Amarok.alert(page);
}

//	--------------------------------------------------------------------------------------------------------------------
//	stores (in some cases calculate) info about searching
function SearchInfo(artist, title, url, ignoreCache){
//	default replaces for artist and title in search url
	this.getDefaultReplaces();

	this.artist = artist;
	this.title = title;
	this.url = url.decodeSuggestionURL();
	this.track = Amarok.Engine.currentTrack();

	this.isValid = this.init();
	if (url!="" && url.indexOf("http")<0 && url.indexOf("www")<0){
		Amarok.alert("url  " + url);
		Amarok.Lyrics.showLyricsError('invalid url "' + url + '"');
		this.isValid = false;			//	invalid url
	}

	this.artist = this.artist.multiReplace(this.defaultReplaces);
	this.title = this.title.multiReplace(this.defaultReplaces);
}

SearchInfo.prototype.getDefaultReplaces = function(){
	SearchInfo.prototype.defaultReplaces = [
settings.omit_parenthesises?	{expr: /\(.*\)/g , str: ""} : undefined,	//	removes parenthesises
settings.omit_brackets?		{expr: /\[.*\]/g , str: ""} : undefined,	//	removes brackets
settings.omit_trach_number?	{expr: /^\d* *[\.\-] */g , str: ""} : undefined,//	removes track number ie. 01.Xyz or 04 - Xyz
settings.omit_amp?		{expr: /&/g , str: ""} : undefined,		//	removes "&"
settings.replace_amp?		{expr: / +& +/g , str: " and "} : undefined,	//	changes "&" to "and"
				{expr: /\ f(ea)?t(uring)?\.?\ .*/mig , str: ""},//	removes "feat" "featuring" ...
				{expr: / *$/g , str: ""},			//	removes ending spaces
				{expr: /\.[\w\d]{2,4}$/gi , str: ""},		//	removes file extensions .mp3 .flac .wav ...
				{expr: /^ */g , str: ""},			//	removes leading spaces
settings.omit_the?		{expr: /^the\ /mig , str: ""} : undefined,	//	removes "the"
	];
}

SearchInfo.prototype.toString = function(){
	return "artist = " + this.artist + "| title = " + this.title + "| url = " + this.url + "| valid = " + this.isValid;
}

SearchInfo.prototype.init = function(){
	if (this.artist == "Unknown" && this.initFileName(this.title))
		return true;

	if (this.artist == "" || this.title == ""){
		var fileName = this.title!=""?this.title:this.artist;
		if (this.initFileName(fileName))
			return true;
		var fileName = (new QFileInfo(this.track.path)).baseName();
		return this.initFileName(fileName);
	}

	if (this.title.length > this.artist.length){
		var artistInTitle = new RegExp(" *"+this.artist+" *- *","i");
		this.title = this.title.replace(artistInTitle,"");	//	tries to remove artist from title
	}
	else if (this.title.length < this.artist.length){
		var titleInArtist = new RegExp(" *- *"+this.title+" *","i");
		this.artist = this.artist.replace(titleInArtist,"");	//	tries to remove title from artist
	}
	return true;
}

SearchInfo.prototype.initFileName = function(fileName){
	if (!arguments.callee.re)
		arguments.callee.re = /^[\d]* *[\.\-]? *(.*) *- *[\d]* *[\.\-]?(.*)/i
	fileName = fileName.replace(/_+/g," ");
//	alert(fileName);
//	var matchFormat = fileName.match(/^[\d]* *[\.\-]? *([^-|( -)]*) *- *[\d]* *[\.\-]?(.*)/i);
	var matchFormat = fileName.match(arguments.callee.re);
	if (matchFormat != undefined){
		this.artist = matchFormat[1];
		this.title = matchFormat[2];
		return true;
	}

	if (fileName.indexOf("-")<0)
		return false;

	fileName = fileName.split(/ *- */);
	if (fileName.length == 0)
		return false;
	this.artist = fileName[0];
	this.title = fileName[1];
	return true;
}

SearchInfo.prototype.isEqual = function(info){
	return this.artist == info.artist && this.title == info.title;
}

SearchInfo.prototype.arrange = function(st,replaces,lowerCase){
	if (!replaces)
		replaces = [];
	if (lowerCase)
		st = st.toLowerCase();
	else
		st = st.capitalize();
	st = st.multiReplace(replaces);
	return st;
}

SearchInfo.prototype.getFetchedTime = function(){
	return (this.timestamp2.getTime()-this.timestamp1.getTime())/1000;
}

/*
var s = new SearchInfo("aristX","01. aristX - titleY","");
alert(s);
var s = new SearchInfo("01. artistA-titleB","titleB","");
alert(s);
*/

//	--------------------------------------------------------------------------------------------------------------------

function SettingsControl(key, defaults){
	this.key = key;
	this.defaults = defaults;
}

SettingsControl.prototype.init= function(){
	return Amarok.Script.readConfig(this.key,JSON.pack(this.defaults)).unpack();
}

SettingsControl.prototype.restoreDefaults= function(){
	var settings = JSON.pack(this.defaults).unpack();
	this.save(settings);
	return settings;
}

SettingsControl.prototype.save= function(settings){
	Amarok.Script.writeConfig(this.key,JSON.pack(settings));
}

String.prototype.unpack = function(type){
	if (this == "")	return undefined;
	return eval("("+this+")");
}

JSON.pack = function(value,type){
	return JSON.stringify(value,null,"\t");
}

//	--------------------------------------------------------------------------------------------------------------------
//	manages multiple sites
SiteManager = {
	//	lock for multiple calls of listener
	working: false,

	setWorking: function(working){
		this.working = working;
		searchGUI.changeSite(true);
	},

	changeSettingsLocation: "Settings->" + qsTranslate("SettingsGUI","Ultimate Lyrics..."),

	getSiteDescriptor: function(site){
		if (siteDescriptors[site])
			return siteDescriptors[site];
		switch(site){
			case "Universal":return new UniversalSite();
			case "google.com":return new GLyrics();
			case "NotFound":return new NotFound();
			default: return undefined;
//			default: throw new Error("there is no descriptor for site '" + site+"'");
		}
	},

	extractDomain: function(site){
		return site.replace(/ +\(.*\)/g,"");
	},

	init: function(newInstall){
		this.currentSiteIndex = 0;
		this.sites = [];
		this.domains = [];
		this.sitesNames = [];

		//	initialize settings
		if (newInstall)		//	restores defaults on every start up for older versions
			settings = config.restoreDefaults();
		else
			settings = config.init();

		//	create cache
		this.fileCache = new FileCache();

		if (newInstall)
			this.fileCache.clear();
		else
			this.fileCache.clearExpired();

		if (settings.cache == "no cache")
			LyricsClass.prototype.cache = new NoCache();
		if (settings.cache == "file cache")
			LyricsClass.prototype.cache = this.fileCache;

		//	load sites
		for(var i in settings.sites){
//			alert("i " + i);
			var site = new LyricsClass();
			var desc = this.getSiteDescriptor(i);
			if (!desc)
				continue;
			site.join(desc);
			site.setup();
			site.init();
			site.skipSearch = !settings.sites[i]["search"];
//			site.useCache = settings.sites[i].cache;
			site.domain = this.extractDomain(i);	//	i
			site.site = i;
			this.sites[this.sites.length]= site;
			if (i.indexOf(".")>0/*site.title*/){
				var url = site.url;
				if (url == undefined)
					url = "http://www." + site.domain;
				url = new QUrl(url).host();
//				alert("site " + site.site);
				this.domains[url] = site;
			}
			this.sitesNames[this.sitesNames.length] = i;
//			Amarok.alert(i);
		}

		//	welcome message
		if (newInstall){
			//:	{version} may not be translated
			var welcomeMessage = qsTranslate("lyrics","Ultimate Lyrics {version} has been installed\ndefault settings are applied and can be changed in:");
			Amarok.alert(welcomeMessage.replace("{version}",settings.version) + "\n" + this.changeSettingsLocation);
		}
	},

	processReq: function(){
		if (this.pendingRequest){
			if (!(this.currentRequest && this.currentRequest.isEqual(this.pendingRequest) && this.currentSiteIndex<this.sitesNames.length && settings.rotating_mode))
				this.currentSiteIndex = 0;
			this.currentRequest = this.pendingRequest;
			this.pendingRequest = undefined;
		}
		if (!this.currentRequest)
			return false;

		if (this.currentSiteIndex == this.sitesNames.length)
			return false;
		this.currentSite = this.sites[this.currentSiteIndex++];
		searchGUI.changeSite(this.currentSite.domain, "searching");
		this.currentSite.processRequest(function(reply){SiteManager.processReply(reply);},this.currentRequest);
		return true;
	},

	//	amarok lyrics listener
	fetchLyrics: function(info) {
		this.pendingRequest = info;
//		searchGUI.update(this.pendingRequest);
		if (!this.working){
			this.setWorking(true);
			if (!this.processReq())
				this.setWorking(false);
		}
	},

	//	handler for receiving responses from 
	processReply : function(reply){
		try{
			if (this.currentSite.processReply(reply,this.currentRequest) || !this.processReq()){
				this.setWorking(false);
			}
		} catch(err) {
			this.setWorking(false);
			alert( 'SiteManager.processReply: (' + this.sitesNames[this.currentSiteIndex-1] + ")" + err );
		}
	}
}

//	--------------------------------------------------------------------------------------------------------------------
//	register listener for amarok lyrics
Amarok.Lyrics.fetchLyrics.connect(function(artist, title, url, ignoreCache) {
	try {
		var info = new SearchInfo(artist, title, url);
//		alert(SiteManager.fetchLyrics);
		SiteManager.fetchLyrics(info);
	} catch(err) {
		this.setWorking(false);
		alert( 'Amarok.Lyrics.fetchLyrics: '  + err );
	}
});
